Package: alakazam
Type: Package
Version: 1.4.3
Date: 2026-04-29
Authors@R: c(person("Susanna", "Marquez", role=c("cre", "aut"), 
                    email="susanna.marquez@yale.edu"),
             person("Namita", "Gupta", role=c("aut"), 
                    email="namita.gupta@yale.edu"),
             person("Nima", "Nouri", role=c("aut"), 
                    email="nima.nouri@yale.edu"),
             person("Ruoyi", "Jiang", role=c("aut"), 
                    email="ruoyi.jiang@yale.edu"),
             person("Julian", "Zhou", role=c("aut"), 
                    email="julian.zhou@bulldogs.yale.edu"),
             person("Kenneth", "Hoehn", role=c("aut"), 
                    email="kenneth.hoehn@yale.edu"),
             person("Daniel", "Gadala-Maria", role=c("ctb"), 
                    email="daniel.gadala-maria@yale.edu"),
             person("Edel", "Aron", role=c("ctb"), 
                    email="edel.aron@yale.edu"),
             person("Cole", "Jensen", role=c("aut"),
                    email="cole.jensen@yale.edu"),
             person("Gisela", "Gabernet", role=c("ctb"),
                    email="gisela.gabernet@yale.edu"),   
             person("Caroline", "Sullivan", role=c("ctb"),
                    email="caroline.sullivan@yale.edu"),
             person("Hailong", "Meng", role=c("ctb"),
                    email="hailong.meng@yale.edu"),
             person("Huimin", "Lyu", role=c("ctb"),
                    email="huimin.lyu@yale.edu"),
             person("Burhan", "Sabuwala", role=c("ctb"),
                    email="burhan.sabuwala@yale.edu"),
             person("Jason", "Vander Heiden", role=c("aut"), 
                    email="jason.vanderheiden@gmail.com"), 
             person("Steven", "Kleinstein", role=c("aut", "cph"), 
                    email="steven.kleinstein@yale.edu"))
Title: Immunoglobulin Clonal Lineage and Diversity Analysis
Description: Provides methods for high-throughput adaptive immune 
    receptor repertoire sequencing (AIRR-Seq; Rep-Seq) analysis. In 
    particular, immunoglobulin (Ig) sequence lineage reconstruction, 
    lineage topology analysis, diversity profiling, amino acid property 
    analysis and gene usage.
    Citations: 
    Gupta and Vander Heiden, et al (2017) <doi:10.1093/bioinformatics/btv359>,
    Stern, Yaari and Vander Heiden, et al (2014) <doi:10.1126/scitranslmed.3008879>.
License: AGPL-3
URL: https://alakazam.readthedocs.io/
BugReports: https://github.com/immcantation/alakazam/issues
LazyData: true
BuildVignettes: true
VignetteBuilder: knitr, rmarkdown
Encoding: UTF-8
LinkingTo: Rcpp
biocViews: Software, AnnotationData
Depends: R (>= 4.0), ggplot2 (>= 3.4.0)
Imports: airr (>= 1.4.1), ape, dplyr (>= 1.0), graphics, grid, igraph
        (>= 1.5.0), Matrix (>= 1.3-0), methods, progress, Rcpp (>=
        0.12.12), readr, rlang, scales, seqinr, stats, stringi, tibble,
        tidyr (>= 1.0), utils, Biostrings (>= 2.56.0),
        GenomicAlignments (>= 1.24.0), IRanges (>= 2.22.2)
Suggests: cigarillo, knitr, rmarkdown, testthat
RoxygenNote: 7.3.3
Collate: 'Alakazam.R' 'AminoAcids.R' 'Classes.R' 'Core.R' 'Data.R'
        'Diversity.R' 'Deprecated.R' 'Fastq.R' 'Gene.R' 'Lineage.R'
        'RcppExports.R' 'Sequence.R' 'Topology.R'
NeedsCompilation: yes
Packaged: 2026-04-29 13:38:10 UTC; root
Author: Susanna Marquez [cre, aut],
  Namita Gupta [aut],
  Nima Nouri [aut],
  Ruoyi Jiang [aut],
  Julian Zhou [aut],
  Kenneth Hoehn [aut],
  Daniel Gadala-Maria [ctb],
  Edel Aron [ctb],
  Cole Jensen [aut],
  Gisela Gabernet [ctb],
  Caroline Sullivan [ctb],
  Hailong Meng [ctb],
  Huimin Lyu [ctb],
  Burhan Sabuwala [ctb],
  Jason Vander Heiden [aut],
  Steven Kleinstein [aut, cph]
Maintainer: Susanna Marquez <susanna.marquez@yale.edu>
Repository: CRAN
Date/Publication: 2026-04-29 15:00:03 UTC
